#!/usr/bin/env python
"""
Parse (transport-layer specific) command line parameters
and create a XCP master instance.
"""

import warnings
from dataclasses import dataclass
from typing import List

from pyxcp.config import create_application, reset_application  # noqa: F401
from pyxcp.master import Master


warnings.simplefilter("always")


@dataclass
class Option:
    short_opt: str
    long_opt: str = ""
    dest: str = ""
    help: str = ""
    type: str = ""
    default: str = ""


class FakeParser:

    options: List[Option] = []

    def add_argument(self, short_opt: str, long_opt: str = "", dest: str = "", help: str = "", type: str = "", default: str = ""):
        warnings.warn("Argument parser extension is currently not supported.", DeprecationWarning, 2)
        self.options.append(Option(short_opt, long_opt, dest, help, type, default))


class ArgumentParser:
    def __init__(self, callout=None, *args, **kws):
        self._parser = FakeParser()
        if callout is not None:
            warnings.warn("callout  argument is currently not supported.", DeprecationWarning, 2)

    def run(self, policy=None, transport_layer_interface=None):
        application = create_application(self.parser.options)
        master = Master(
            application.transport.layer, config=application, policy=policy, transport_layer_interface=transport_layer_interface
        )
        return master

    @property
    def parser(self):
        return self._parser



#!/usr/bin/env python
"""Universal Calibration Protocol for Python."""

from rich import pretty
from rich.console import Console
from rich.traceback import install as tb_install


pretty.install()

from .master import Master  # noqa: F401, E402
from .transport import Can, Eth, SxI, Usb  # noqa: F401, E402


console = Console()
tb_install(show_locals=True, max_frames=3)  # Install custom exception handler.

# if you update this manually, do not forget to update
# .bumpversion.cfg and pyproject.toml.
__version__ = "0.22.18"



#!/usr/bin/env python
"""Fetch A2L file from XCP slave (if supported).
"""
import sys
from pathlib import Path

from rich.prompt import Confirm

from pyxcp.cmdline import ArgumentParser
from pyxcp.types import XcpGetIdType


def main():
    ap = ArgumentParser(description="Fetch A2L file from XCP slave.")

    with ap.run() as x:
        x.connect()

        # TODO: error-handling.
        file_name = x.identifier(XcpGetIdType.FILENAME)
        content = x.identifier(XcpGetIdType.FILE_TO_UPLOAD)
        x.disconnect()
        if not content:
            sys.exit(f"Empty response from ID '{XcpGetIdType.FILE_TO_UPLOAD!r}'.")
        if not file_name:
            file_name = "output.a2l"
        if not file_name.lower().endswith(".a2l"):
            file_name += ".a2l"
        dest = Path(file_name)
        if dest.exists():
            if not Confirm.ask(f"Destination file [green]{dest.name!r}[/green] already exists. Do you want to overwrite it?"):
                print("Aborting...")
                exit(1)
        with dest.open("wt", encoding="utf-8") as of:
            of.write(content)
        print(f"A2L data written to {file_name!r}.")


if __name__ == "__main__":
    main()



import logging
from array import array
from dataclasses import dataclass, field
from typing import Any, List


MAP_TO_ARRAY = {
    "U8": "B",
    "I8": "b",
    "U16": "H",
    "I16": "h",
    "U32": "L",
    "I32": "l",
    "U64": "Q",
    "I64": "q",
    "F32": "f",
    "F64": "d",
    "F16": "f",
    "BF16": "f",
}

logger = logging.getLogger("PyXCP")


@dataclass
class Storage:
    name: str
    arrow_type: Any
    arr: array


@dataclass
class StorageContainer:
    name: str
    arr: List[Storage] = field(default_factory=[])
    ts0: List[int] = field(default_factory=lambda: array("Q"))
    ts1: List[int] = field(default_factory=lambda: array("Q"))



#!/usr/bin/env python
"""Very basic hello-world example.
"""
import time

import matplotlib.pyplot as plt
import seaborn as sns

from pyxcp.cmdline import ArgumentParser
from pyxcp.transport import FrameRecorderPolicy


sns.set()


ADDR = 0x4000
LENGTH = 0x1000
ITERATIONS = 100

recorder_policy = FrameRecorderPolicy()  # Create frame recorder.
ap = ArgumentParser(description="pyXCP hello world.", policy=recorder_policy)
with ap.run() as x:
    xs = []
    ys = []
    x.connect()
    for ctoSize in range(8, 64 + 4, 4):
        print(f"CTO-Size: {ctoSize}")
        xs.append(ctoSize)
        start = time.perf_counter()
        for _ in range(ITERATIONS):
            x.setMta(ADDR)
            data = x.fetch(LENGTH, ctoSize)
        et = time.perf_counter() - start
        ys.append(et)
        print(f"CTO size: {ctoSize:-3} -- elapsed time {et:-3.04}")
    x.disconnect()
    plt.plot(xs, ys)
    plt.show()



import argparse
import logging
from array import array
from dataclasses import dataclass, field
from typing import Any, List

import pyarrow as pa
import pyarrow.parquet as pq

from pyxcp.recorder import XcpLogFileDecoder


MAP_TO_ARROW = {
    "U8": pa.uint8(),
    "I8": pa.int8(),
    "U16": pa.uint16(),
    "I16": pa.int16(),
    "U32": pa.uint32(),
    "I32": pa.int32(),
    "U64": pa.uint64(),
    "I64": pa.int64(),
    "F32": pa.float32(),
    "F64": pa.float64(),
    "F16": pa.float16(),
    "BF16": pa.float16(),
}

MAP_TO_ARRAY = {
    "U8": "B",
    "I8": "b",
    "U16": "H",
    "I16": "h",
    "U32": "L",
    "I32": "l",
    "U64": "Q",
    "I64": "q",
    "F32": "f",
    "F64": "d",
    "F16": "f",
    "BF16": "f",
}

logger = logging.getLogger("PyXCP")

parser = argparse.ArgumentParser(description="Use .xmraw files in an Apache Arrow application.")
parser.add_argument("xmraw_file", help=".xmraw file")
args = parser.parse_args()


@dataclass
class Storage:
    name: str
    arrow_type: Any
    arr: array


@dataclass
class StorageContainer:
    name: str
    arr: List[Storage] = field(default_factory=[])
    ts0: List[int] = field(default_factory=lambda: array("Q"))
    ts1: List[int] = field(default_factory=lambda: array("Q"))


class Decoder(XcpLogFileDecoder):

    def initialize(self) -> None:
        self.arrow_tables = []
        for dl in self.daq_lists:
            result = []
            for name, type_str in dl.headers:
                array_txpe = MAP_TO_ARRAY[type_str]
                arrow_type = MAP_TO_ARROW[type_str]
                sd = Storage(name, arrow_type, array(array_txpe))
                print(f"\t{name!r} {array_txpe} {arrow_type}", sd)
                result.append(sd)
            sc = StorageContainer(dl.name, result)
            self.arrow_tables.append(sc)

    def finalize(self) -> Any:
        result = []
        for arr in self.arrow_tables:
            timestamp0 = arr.ts0
            timestamp1 = arr.ts1
            names = ["timestamp0", "timestamp1"]
            data = [timestamp0, timestamp1]
            for sd in arr.arr:
                adt = pa.array(sd.arr, type=sd.arrow_type)
                names.append(sd.name)
                data.append(adt)
            table = pa.Table.from_arrays(data, names=names)
            fname = f"{arr.name}.parquet"
            print("Writing table", fname)
            pq.write_table(table, fname)
            print("done.", table.shape)
            result.append(table)
        return result

    def on_daq_list(self, daq_list_num: int, timestamp0: int, timestamp1: int, measurements: list) -> None:
        sc = self.arrow_tables[daq_list_num]
        sc.ts0.append(timestamp0)
        sc.ts1.append(timestamp1)
        for idx, elem in enumerate(measurements):
            sto = sc.arr[idx]
            sto.arr.append(elem)


decoder = Decoder(args.xmraw_file)
res = decoder.run()
